-------------------------------------------------------------------------------
--   Projekt   : kniha Cislicove systemy a jazyk VHDL
--   Jm. soub. : mealy.vhd
--   Nazev     : Vypis 10.2
--   Popis     : Konecny automat typu Mealy
-------------------------------------------------------------------------------
--   Datum      Autor             Revize  Poznamka
--   16.8.2006  Martin Poupa      Rev A   Vytvoreno
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity mealy is
   port (
      clk : in std_logic;
      i0, i1, i2 : in std_logic;
      o0, o1, o2 : out std_logic
   );
end mealy;

architecture behav of mealy is
   type stavy is (s0, s1, s2);
   signal stav_reg, dalsi_stav : stavy := s0;
begin
   -- realizace pameti stavu Mealyho automatu
   process (clk)
   begin
      if clk'event and clk = '1' then
         stav_reg <= dalsi_stav;
      end if;
   end process;

   -- prechodova funkce Mealyho automatu
   process (stav_reg, i0, i1, i2)
   begin
      -- prechodove podminky vedouci na jiny stav
      case stav_reg is
         -- stav s0
         when s0 =>
            if i0 = '1' then
               dalsi_stav <= s1;
            else
               dalsi_stav <= s0;
            end if;
         -- stav s1
         when s1 =>
            if i0 = '1' then
               dalsi_stav <= s2;
            elsif i1 = '1' then
               dalsi_stav <= s0;
            else
               dalsi_stav <= s1;
            end if;
         -- stav s2
         when s2 =>
            if i1 = '1' or i2 = '1' then
               dalsi_stav <= s0;
            else
               dalsi_stav <= s2;
            end if;
         -- pri vsech ostatnich stavech zustava v soucasnem stavu
         when others =>
            dalsi_stav <= stav_reg;
      end case;
   end process;

   -- vystupni funkce Mealyho automatu
   process (stav_reg, i0, i1, i2)
   begin
      case stav_reg is
         -- stav s0
         when s0 =>
            o0 <= i0;
            o1 <= i1 or i2;
            o2 <= '0';
         -- stav s1
         when s1 =>
            o0 <= i2;
            o1 <= '0';
            o2 <= i0 or i1;
         -- stav s2
         when s2 =>
            o0 <= i1 or i2;
            o1 <= '0';
            o2 <= i0;
         -- v ostatnich pripadech nedelej nic
         when others => null;
      end case;
   end process;
end behav;
