-------------------------------------------------------------------------------
--   Projekt   : kniha Cislicove systemy a jazyk VHDL
--   Jm. soub. : ram_async.vhd
--   Nazev     : Vypis 12.1
--   Popis     : Asynchronni pamet RAM
-------------------------------------------------------------------------------
--   Datum      Autor             Revize  Poznamka
--   16.8.2006  Martin Poupa      Rev A   Vytvoreno
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ram_async is
   port (
      adr : in std_logic_vector (15 downto 0);
      di : in std_logic_vector (7 downto 0);
      cs : in std_logic;
      oe : in std_logic;
      wr : in std_logic;
      do : out std_logic_vector (7 downto 0)
   );
end ram_async;

architecture behav of ram_async is
   type mem is array (0 to 65535) of std_logic_vector (7 downto 0);
   signal ram_block : mem;
begin
   -- realizace zapisu do ram procesem
   -- proces se spousti pri kazde zmene wr nebo di
   process (wr, di)
   begin
      -- zapis do pameti probiha pouze pri aktivnim cs a wr
      if cs = '0' and wr = '0' then
         ram_block(to_integer(unsigned(adr))) <= di;
      end if;
   end process;

   -- cteni probiha pri aktivnim cs a oe, jinak dava na vysokou impedanci
   do <= ram_block(to_integer(unsigned(adr))) when (cs = '0' and oe = '0')
         else (others => 'Z');
end behav;
